import logging
import typing as t
from logging.config import DictConfigurator

from pydantic_settings import BaseSettings
from pydantic_settings import SettingsConfigDict

from ... import exceptions_
from ... import mixins
from .filters import Filter
from .formatters import Formatter
from .handlers import Handler
from .loggers import Logger


class Log(
    mixins.ByAliasModelDumpMixin,
    mixins.ExcludeNoneModelDumpMixin,
    mixins.DoNotEnsureAsciiModelDumpJsonModel,
    mixins.Indent2ModelDumpJsonModel,
    BaseSettings,
):
    model_config = SettingsConfigDict(
        env_prefix='log__',
        env_nested_delimiter='__',
        extra='ignore',
    )
    # https://docs.python.org/3/library/logging.config.html#configuration-dictionary-schema
    # https://docs.python.org/3/howto/logging-cookbook.html#an-example-dictionary-based-configuration
    version: int = 1  # ok
    formatters: t.Optional[dict[str, Formatter]] = None
    filters: t.Optional[dict[str, Filter]] = None
    handlers: t.Optional[dict[str, Handler]] = None
    loggers: t.Optional[dict[str, Logger]] = None
    root: t.Optional[Logger] = None
    incremental: t.Optional[bool] = None
    disable_existing_loggers: t.Optional[bool] = None

    configurator: t.ClassVar = DictConfigurator

    def _configure(self, configuration):
        self.configurator(configuration).configure()

    def configure(self):
        configuration = self.model_dump()
        self._configure(configuration)

    def is_valid_logger_name(self, logger_name: str):
        return logger_name in ['', 'root', *(self.loggers or {}).keys()]

    def get_logger(self, logger_name: str):
        if not self.is_valid_logger_name(logger_name):
            raise exceptions_.LoggerNotFoundError(logger_name)
        return logging.getLogger(name=logger_name)

    def configure_and_get_logger(self, logger_name: str = ''):
        self.configure()
        return self.get_logger(logger_name=logger_name)
